@@ -37,6 +37,7 @@ import subprocess
 import sys
 import tempfile
 import time
+import stat
 import traceback
 import warnings
 import logging
@@ -101,6 +102,14 @@ def check_missing_parameters(options):
   if missing:
     raise RuntimeError('Missing mandatory parameters: %s' % ', '.join(sorted(missing)))
 
+  # parameter can NOT be empty string or None
+  for option in required:
+    if not options.get(option):
+      missing.add(option)
+
+  if missing:
+    raise RuntimeError('Mandatory parameters present but empty: %s' % ', '.join(sorted(missing)))
+
 
 def check_missing_files(options):
   req_files = [
@@ -352,8 +361,17 @@ class Slapgrid(object):
       raise OSError('%s does not exist.' % self.instance_root)
     # Creates everything needed
 
+    # Create directory accessible for the instances.
+    var_directory = os.path.join(self.instance_root, 'var')
+    if not os.path.isdir(var_directory):
+      os.mkdir(var_directory)
+
+    os.chmod(var_directory, stat.S_IRWXU | stat.S_IROTH | stat.S_IXOTH | \
+                            stat.S_IRGRP | stat.S_IXGRP )
+
+    mkdir_p(os.path.join(self.instance_root, 'var'), 0o755)
+
     # Creates instance_root structure
-    createPrivateDirectory(os.path.join(self.instance_root, 'var'))
     createPrivateDirectory(os.path.join(self.instance_root, 'var', 'log'))
     createPrivateDirectory(os.path.join(self.instance_root, 'var', 'run'))
 
@@ -458,7 +476,7 @@ class Slapgrid(object):
         if state == 'available':
           try:
             software_release.available()
-          except NotFoundError:
+          except (NotFoundError, ServerError):
             pass
         elif state == 'destroyed':
           try:
@@ -615,7 +633,7 @@ class Slapgrid(object):
                 filename="%s/instance.log" % (log_folder_path)
             )
     stat_info = os.stat(instance_path)
-    chownDirectory("%s/.slapgrid" % instance_path, 
+    chownDirectory("%s/.slapgrid" % instance_path,
                    uid=stat_info.st_uid,
                    gid=stat_info.st_gid)
 
@@ -629,7 +647,13 @@ class Slapgrid(object):
       self.logger.info('  Software URL: %s' % software_url)
       self.logger.info('  Software path: %s' % software_path)
       self.logger.info('  Instance path: %s' % instance_path)
-  
+
+      filter_dict = getattr(computer_partition, '_filter_dict', None)
+      if filter_dict:
+        retention_delay = filter_dict.get('retention_delay', '0')
+      else:
+        retention_delay = '0'
+
       local_partition = Partition(
         software_path=software_path,
         instance_path=instance_path,
@@ -644,13 +668,15 @@ class Slapgrid(object):
         software_release_url=software_url,
         certificate_repository_path=self.certificate_repository_path,
         buildout=self.buildout,
-        logger=self.logger)
+        logger=self.logger,
+        retention_delay=retention_delay,
+      )
       computer_partition_state = computer_partition.getState()
-  
+
       # XXX this line breaks 37 tests
       # self.logger.info('  Instance type: %s' % computer_partition.getType())
       self.logger.info('  Instance status: %s' % computer_partition_state)
-  
+
       if computer_partition_state == COMPUTER_PARTITION_STARTED_STATE:
         local_partition.install()
         computer_partition.available()
@@ -974,9 +1000,9 @@ class Slapgrid(object):
         computer_partition_id = computer_partition.getId()
         instance_path = os.path.join(self.instance_root, computer_partition_id)
         dir_report_list = [os.path.join(instance_path, 'var', 'xml_report'),
-            os.path.join(self.instance_root, 'var', 'xml_report', 
+            os.path.join(self.instance_root, 'var', 'xml_report',
                          computer_partition_id)]
-        
+
         for dir_reports in dir_report_list:
           # The directory xml_report contain a number of files equal
           # to the number of software instance running inside the same partition
@@ -1070,7 +1096,8 @@ class Slapgrid(object):
             software_release_url=software_url,
             certificate_repository_path=self.certificate_repository_path,
             buildout=self.buildout,
-            logger=self.logger)
+            logger=self.logger,
+          )
           local_partition.stop()
           try:
             computer_partition.stopped()
@@ -1083,7 +1110,7 @@ class Slapgrid(object):
             self.logger.info('Ignoring destruction of %r, as no report usage was sent' %
                                 computer_partition.getId())
             continue
-          local_partition.destroy()
+          destroyed = local_partition.destroy()
         except (SystemExit, KeyboardInterrupt):
           computer_partition.error(traceback.format_exc(), logger=self.logger)
           raise
@@ -1093,7 +1120,8 @@ class Slapgrid(object):
           exc = traceback.format_exc()
           computer_partition.error(exc, logger=self.logger)
         try:
-          computer_partition.destroyed()
+          if destroyed:
+            computer_partition.destroyed()
         except NotFoundError:
           self.logger.debug('Ignored slap error while trying to inform about '
                             'destroying not fully configured Computer Partition %r' %
