@@ -31,23 +31,33 @@ Simple, easy to (un)marshall classes for slap client/server communication
 """
 
 __all__ = ["slap", "ComputerPartition", "Computer", "SoftwareRelease",
+           "SoftwareProductCollection",
            "Supply", "OpenOrder", "NotFoundError", "Unauthorized",
            "ResourceNotReady", "ServerError"]
 
-from interface import slap as interface
-from xml_marshaller import xml_marshaller
 import httplib
 import logging
+import re
 import socket
 import ssl
-import traceback
 import urllib
 import urlparse
+from util import xml2dict
+
+from xml.sax import saxutils
 import zope.interface
+from interface import slap as interface
+from xml_marshaller import xml_marshaller
 
-log = logging.getLogger(__name__)
+# XXX fallback_logger to be deprecated together with the old CLI entry points.
+fallback_logger = logging.getLogger(__name__)
+fallback_handler = logging.StreamHandler()
+fallback_logger.setLevel(logging.INFO)
+fallback_logger.addHandler(fallback_handler)
+
+
+DEFAULT_SOFTWARE_TYPE = 'RootSoftwareInstance'
 
-DEFAULT_SOFTWARE_TYPE = 'default'
 
 # httplib.HTTPSConnection with key verification
 class HTTPSConnectionCA(httplib.HTTPSConnection):
@@ -77,11 +87,42 @@ class SlapDocument:
       # cause failures when accessing _connection_helper property.
       self._connection_helper = connection_helper
 
+
+class SlapRequester(SlapDocument):
+  """
+  Abstract class that allow to factor method for subclasses that use "request()"
+  """
+  def _requestComputerPartition(self, request_dict):
+    try:
+      xml = self._connection_helper.POST('/requestComputerPartition', request_dict)
+    except ResourceNotReady:
+      return ComputerPartition(
+        request_dict=request_dict,
+        connection_helper=self._connection_helper,
+      )
+    software_instance = xml_marshaller.loads(xml)
+    computer_partition = ComputerPartition(
+      software_instance.slap_computer_id.encode('UTF-8'),
+      software_instance.slap_computer_partition_id.encode('UTF-8'),
+      connection_helper=self._connection_helper,
+    )
+    # Hack to give all object attributes to the ComputerPartition instance
+    # XXX Should be removed by correctly specifying difference between
+    # ComputerPartition and SoftwareInstance
+    computer_partition.__dict__ = dict(computer_partition.__dict__.items() +
+                                       software_instance.__dict__.items())
+    # XXX not generic enough.
+    if xml_marshaller.loads(request_dict['shared_xml']):
+      computer_partition._synced = True
+      computer_partition._connection_dict = software_instance._connection_dict
+      computer_partition._parameter_dict = software_instance._parameter_dict
+    return computer_partition
+
+
 class SoftwareRelease(SlapDocument):
   """
   Contains Software Release information
   """
-
   zope.interface.implements(interface.ISoftwareRelease)
 
   def __init__(self, software_release=None, computer_guid=None, **kw):
@@ -112,16 +153,15 @@ class SoftwareRelease(SlapDocument):
     else:
       return self._software_release
 
-  def error(self, error_log):
+  def error(self, error_log, logger=None):
     try:
       # Does not follow interface
       self._connection_helper.POST('/softwareReleaseError', {
         'url': self.getURI(),
-        'computer_id' : self.getComputerId(),
+        'computer_id': self.getComputerId(),
         'error_log': error_log})
     except Exception:
-      exception = traceback.format_exc()
-      log.error(exception)
+      (logger or fallback_logger).exception('')
 
   def available(self):
     self._connection_helper.POST('/availableSoftwareRelease', {
@@ -141,6 +181,28 @@ class SoftwareRelease(SlapDocument):
   def getState(self):
     return getattr(self, '_requested_state', 'available')
 
+
+class SoftwareProductCollection(object):
+  zope.interface.implements(interface.ISoftwareProductCollection)
+
+  def __init__(self, logger, slap):
+    self.logger = logger
+    self.slap = slap
+    self.__getattr__ = self.get
+  def get(self, software_product):
+      self.logger.info('Getting best Software Release corresponging to '
+                       'this Software Product...')
+      software_release_list = \
+          self.slap.getSoftwareReleaseListFromSoftwareProduct(software_product)
+      try:
+          software_release_url = software_release_list[0] # First is best one.
+          self.logger.info('Found as %s.' % software_release_url)
+          return software_release_url
+      except IndexError:
+          raise AttributeError('No Software Release corresponding to this '
+                           'Software Product has been found.')
+
+
 # XXX What is this SoftwareInstance class?
 class SoftwareInstance(SlapDocument):
   """
@@ -154,22 +216,28 @@ class SoftwareInstance(SlapDocument):
     for k, v in kwargs.iteritems():
       setattr(self, k, v)
 
+
 """Exposed exceptions"""
+
+
 # XXX Why do we need to expose exceptions?
 class ResourceNotReady(Exception):
   zope.interface.implements(interface.IResourceNotReady)
 
+
 class ServerError(Exception):
   zope.interface.implements(interface.IServerError)
 
+
 class NotFoundError(Exception):
   zope.interface.implements(interface.INotFoundError)
 
+
 class Unauthorized(Exception):
   zope.interface.implements(interface.IUnauthorized)
 
-class Supply(SlapDocument):
 
+class Supply(SlapDocument):
   zope.interface.implements(interface.ISupply)
 
   def supply(self, software_release, computer_guid=None, state='available'):
@@ -182,13 +250,13 @@ class Supply(SlapDocument):
       raise NotFoundError("Computer %s has not been found by SlapOS Master."
           % computer_guid)
 
-class OpenOrder(SlapDocument):
 
+class OpenOrder(SlapRequester):
   zope.interface.implements(interface.IOpenOrder)
 
   def request(self, software_release, partition_reference,
-      partition_parameter_kw=None, software_type=None, filter_kw=None,
-      state=None, shared=False):
+              partition_parameter_kw=None, software_type=None,
+              filter_kw=None, state=None, shared=False):
     if partition_parameter_kw is None:
       partition_parameter_kw = {}
     if filter_kw is None:
@@ -202,44 +270,25 @@ class OpenOrder(SlapDocument):
         #             And second is a boolean.
         'state': xml_marshaller.dumps(state),
         'shared_xml': xml_marshaller.dumps(shared),
-      }
+    }
     if software_type is not None:
       request_dict['software_type'] = software_type
     else:
       # Let's enforce a default software type
       request_dict['software_type'] = DEFAULT_SOFTWARE_TYPE
-    try:
-      self._connection_helper.POST('/requestComputerPartition', request_dict)
-    except ResourceNotReady:
-      return ComputerPartition(
-        request_dict=request_dict,
-        connection_helper=self._connection_helper,
-      )
-    else:
-      xml = self._connection_helper.response.read()
-      software_instance = xml_marshaller.loads(xml)
-      computer_partition = ComputerPartition(
-        software_instance.slap_computer_id.encode('UTF-8'),
-        software_instance.slap_computer_partition_id.encode('UTF-8'),
-        connection_helper=self._connection_helper,
-      )
-      if shared:
-        computer_partition._synced = True
-        computer_partition._connection_dict = software_instance._connection_dict
-        computer_partition._parameter_dict = software_instance._parameter_dict
-      return computer_partition
+    return self._requestComputerPartition(request_dict)
 
   def requestComputer(self, computer_reference):
     """
     Requests a computer.
     """
-    self._connection_helper.POST('/requestComputer',
+    xml = self._connection_helper.POST('/requestComputer',
       {'computer_title': computer_reference})
-    xml = self._connection_helper.response.read()
     computer = xml_marshaller.loads(xml)
     computer._connection_helper = self._connection_helper
     return computer
 
+
 def _syncComputerInformation(func):
   """
   Synchronize computer object with server information
@@ -260,8 +309,8 @@ def _syncComputerInformation(func):
     return func(self, *args, **kw)
   return decorated
 
-class Computer(SlapDocument):
 
+class Computer(SlapDocument):
   zope.interface.implements(interface.IComputer)
 
   def __init__(self, computer_id, connection_helper=None):
@@ -287,7 +336,7 @@ class Computer(SlapDocument):
   def getComputerPartitionList(self):
     for computer_partition in self._computer_partition_list:
       computer_partition._connection_helper = self._connection_helper
-    return [x for x in self._computer_partition_list ]
+    return [x for x in self._computer_partition_list]
 
   def reportUsage(self, computer_usage):
     if computer_usage == "":
@@ -297,9 +346,8 @@ class Computer(SlapDocument):
       'use_string': computer_usage})
 
   def updateConfiguration(self, xml):
-    self._connection_helper.POST(
-        '/loadComputerConfigurationFromXML', { 'xml' : xml })
-    return self._connection_helper.response.read()
+    return self._connection_helper.POST(
+        '/loadComputerConfigurationFromXML', {'xml': xml})
 
   def bang(self, message):
     self._connection_helper.POST('/computerBang', {
@@ -307,78 +355,38 @@ class Computer(SlapDocument):
       'message': message})
 
   def getStatus(self):
-    self._connection_helper.GET(
+    xml = self._connection_helper.GET(
         '/getComputerStatus?computer_id=%s' % self._computer_id)
-    return xml_marshaller.loads(self._connection_helper.response.read())
+    return xml_marshaller.loads(xml)
 
   def revokeCertificate(self):
     self._connection_helper.POST('/revokeComputerCertificate', {
       'computer_id': self._computer_id})
 
   def generateCertificate(self):
-    self._connection_helper.POST('/generateComputerCertificate', {
+    xml = self._connection_helper.POST('/generateComputerCertificate', {
       'computer_id': self._computer_id})
-    xml = self._connection_helper.response.read()
     return xml_marshaller.loads(xml)
 
-def _syncComputerPartitionInformation(func):
-  """
-  Synchronize computer partition object with server information
-  """
-  def decorated(self, *args, **kw):
-    if getattr(self, '_synced', 0):
-      return func(self, *args, **kw)
-    if not self._computer_id:
-      # XXX Is it only in case of requesting instance?
-      raise ResourceNotReady("Instance is not ready yet.")
-    # XXX: This is a ugly way to keep backward compatibility,
-    # We should stablise slap library soon.
-    computer = self._connection_helper.getFullComputerInformation(self._computer_id)
-    found_computer_partition = None
-    for computer_partition in computer._computer_partition_list:
-      if computer_partition.getId() == self.getId():
-        found_computer_partition = computer_partition
-        break
-    if found_computer_partition is None:
-      raise NotFoundError("No information for partition %s" %
-          self.getId())
-    else:
-      for key, value in found_computer_partition.__dict__.items():
-        if isinstance(value, unicode):
-          # convert unicode to utf-8
-          setattr(self, key, value.encode('utf-8'))
-        if isinstance(value, dict):
-          new_dict = {}
-          for ink, inv in value.iteritems():
-            if isinstance(inv, (list, tuple)):
-              new_inv = []
-              for elt in inv:
-                if isinstance(elt, (list, tuple)):
-                  new_inv.append([x.encode('utf-8') for x in elt])
-                elif isinstance(elt, dict):
-                  new_inv.append(dict([(x.encode('utf-8'),
-                    y and y.encode("utf-8")) for x,y in elt.iteritems()]))
-                else:
-                  new_inv.append(elt.encode('utf-8'))
-              new_dict[ink.encode('utf-8')] = new_inv
-            elif inv is None:
-              new_dict[ink.encode('utf-8')] = None
-            else:
-              new_dict[ink.encode('utf-8')] = inv.encode('utf-8')
-          setattr(self, key, new_dict)
-        else:
-          setattr(self, key, value)
-    setattr(self, '_synced', True)
-    return func(self, *args, **kw)
-  return decorated
+
+def parsed_error_message(status, body, path):
+  m = re.search('(Error Value:\n.*)', body, re.MULTILINE)
+  if m:
+    match = ' '.join(line.strip() for line in m.group(0).split('\n'))
+    return '%s (status %s while calling %s)' % (
+                saxutils.unescape(match),
+                status,
+                path
+            )
+  else:
+    return 'Server responded with wrong code %s with %s' % (status, path)
 
 
-class ComputerPartition(SlapDocument):
-
+class ComputerPartition(SlapRequester):
   zope.interface.implements(interface.IComputerPartition)
 
-  def __init__(self, computer_id=None, partition_id=None, request_dict=None,
-      connection_helper=None):
+  def __init__(self, computer_id=None, partition_id=None,
+               request_dict=None, connection_helper=None):
     SlapDocument.__init__(self, connection_helper)
     if request_dict is not None and (computer_id is not None or
         partition_id is not None):
@@ -394,27 +402,27 @@ class ComputerPartition(SlapDocument):
   def __getinitargs__(self):
     return (self._computer_id, self._partition_id, )
 
-  @_syncComputerPartitionInformation
   def request(self, software_release, software_type, partition_reference,
               shared=False, partition_parameter_kw=None, filter_kw=None,
               state=None):
     if partition_parameter_kw is None:
       partition_parameter_kw = {}
     elif not isinstance(partition_parameter_kw, dict):
-      raise ValueError("Unexpected type of partition_parameter_kw '%s'" % \
+      raise ValueError("Unexpected type of partition_parameter_kw '%s'" %
                        partition_parameter_kw)
 
     if filter_kw is None:
       filter_kw = {}
     elif not isinstance(filter_kw, dict):
-      raise ValueError("Unexpected type of filter_kw '%s'" % \
+      raise ValueError("Unexpected type of filter_kw '%s'" %
                        filter_kw)
 
     # Let enforce a default software type
     if software_type is None:
       software_type = DEFAULT_SOFTWARE_TYPE
 
-    request_dict = { 'computer_id': self._computer_id,
+    request_dict = {
+        'computer_id': self._computer_id,
         'computer_partition_id': self._partition_id,
         'software_release': software_release,
         'software_type': software_type,
@@ -424,29 +432,8 @@ class ComputerPartition(SlapDocument):
                                         partition_parameter_kw),
         'filter_xml': xml_marshaller.dumps(filter_kw),
         'state': xml_marshaller.dumps(state),
-      }
-    try:
-      self._connection_helper.POST('/requestComputerPartition', request_dict)
-    except ResourceNotReady:
-      return ComputerPartition(
-        request_dict=request_dict,
-        connection_helper=self._connection_helper,
-      )
-    else:
-      xml = self._connection_helper.response.read()
-      software_instance = xml_marshaller.loads(xml)
-      computer_partition = ComputerPartition(
-        software_instance.slap_computer_id.encode('UTF-8'),
-        software_instance.slap_computer_partition_id.encode('UTF-8'),
-        connection_helper=self._connection_helper,
-      )
-      if shared:
-        computer_partition._synced = True
-        computer_partition._connection_dict = getattr(software_instance,
-          '_connection_dict', None)
-        computer_partition._parameter_dict = getattr(software_instance,
-          '_parameter_dict', None)
-      return computer_partition
+    }
+    return self._requestComputerPartition(request_dict)
 
   def building(self):
     self._connection_helper.POST('/buildingComputerPartition', {
@@ -476,15 +463,14 @@ class ComputerPartition(SlapDocument):
       'computer_partition_id': self.getId(),
       })
 
-  def error(self, error_log):
+  def error(self, error_log, logger=None):
     try:
       self._connection_helper.POST('/softwareInstanceError', {
         'computer_id': self._computer_id,
         'computer_partition_id': self.getId(),
         'error_log': error_log})
     except Exception:
-      exception = traceback.format_exc()
-      log.error(exception)
+      (logger or fallback_logger).exception('')
 
   def bang(self, message):
     self._connection_helper.POST('/softwareInstanceBang', {
@@ -493,43 +479,55 @@ class ComputerPartition(SlapDocument):
       'message': message})
 
   def rename(self, new_name, slave_reference=None):
-    post_dict = dict(
-      computer_id=self._computer_id,
-      computer_partition_id=self.getId(),
-      new_name=new_name,
-    )
-    if slave_reference is not None:
-      post_dict.update(slave_reference=slave_reference)
+    post_dict = {
+            'computer_id': self._computer_id,
+            'computer_partition_id': self.getId(),
+            'new_name': new_name,
+            }
+    if slave_reference:
+      post_dict['slave_reference'] = slave_reference
     self._connection_helper.POST('/softwareInstanceRename', post_dict)
 
   def getId(self):
-    if not self._partition_id:
+    if not getattr(self, '_partition_id', None):
       raise ResourceNotReady()
     return self._partition_id
 
-  @_syncComputerPartitionInformation
   def getInstanceGuid(self):
-    """Sync if not synced, then returns instance_guid"""
+    """Return instance_guid. Raise ResourceNotReady if it doesn't exist."""
     if not getattr(self, '_instance_guid', None):
       raise ResourceNotReady()
     return self._instance_guid
 
-  @_syncComputerPartitionInformation
   def getState(self):
-    """Sync if not synced, then returns _requested_state."""
+    """return _requested_state. Raise ResourceNotReady if it doesn't exist."""
     if not getattr(self, '_requested_state', None):
       raise ResourceNotReady()
     return self._requested_state
 
-  @_syncComputerPartitionInformation
+  def getType(self):
+    """
+    return the Software Type of the instance.
+    Raise RessourceNotReady if not present.
+    """
+    # XXX: software type should not belong to the parameter dict.
+    software_type = self.getInstanceParameterDict().get(
+        'slap_software_type', None)
+    if not software_type:
+      raise ResourceNotReady()
+    return software_type
+
   def getInstanceParameterDict(self):
     return getattr(self, '_parameter_dict', None) or {}
 
-  @_syncComputerPartitionInformation
   def getConnectionParameterDict(self):
-    return getattr(self, '_connection_dict', None) or {}
+    connection_dict = getattr(self, '_connection_dict', None)
+    if connection_dict is None:
+      # XXX Backward compatibility for older slapproxy (<= 1.0.0)
+      connection_dict = xml2dict(getattr(self, 'connection_xml', '')) 
 
-  @_syncComputerPartitionInformation
+    return connection_dict or {}
+      
   def getSoftwareRelease(self):
     """
     Returns the software release associate to the computer partition.
@@ -548,7 +546,6 @@ class ComputerPartition(SlapDocument):
           'connection_xml': xml_marshaller.dumps(connection_dict),
           'slave_reference': slave_reference})
 
-  @_syncComputerPartitionInformation
   def getInstanceParameter(self, key):
     parameter_dict = getattr(self, '_parameter_dict', None) or {}
     if key in parameter_dict:
@@ -556,9 +553,8 @@ class ComputerPartition(SlapDocument):
     else:
       raise NotFoundError("%s not found" % key)
 
-  @_syncComputerPartitionInformation
   def getConnectionParameter(self, key):
-    connection_dict = getattr(self, '_connection_dict', None) or {}
+    connection_dict = self.getConnectionParameterDict()
     if key in connection_dict:
       return connection_dict[key]
     else:
@@ -569,16 +565,17 @@ class ComputerPartition(SlapDocument):
     self.usage = usage_log
 
   def getCertificate(self):
-    self._connection_helper.GET(
+    xml = self._connection_helper.GET(
         '/getComputerPartitionCertificate?computer_id=%s&'
         'computer_partition_id=%s' % (self._computer_id, self._partition_id))
-    return xml_marshaller.loads(self._connection_helper.response.read())
+    return xml_marshaller.loads(xml)
 
   def getStatus(self):
-    self._connection_helper.GET(
+    xml = self._connection_helper.GET(
         '/getComputerPartitionStatus?computer_id=%s&'
         'computer_partition_id=%s' % (self._computer_id, self._partition_id))
-    return xml_marshaller.loads(self._connection_helper.response.read())
+    return xml_marshaller.loads(xml)
+
 
 class ConnectionHelper:
   error_message_timeout = "\nThe connection timed out. Please try again later."
@@ -588,8 +585,9 @@ class ConnectionHelper:
       "original error was: "
   ssl_error_message_connect_fail = "\nCouldn't authenticate computer. Please "\
       "check that certificate and key exist and are valid. "
+
   def __init__(self, connection_wrapper, host, path, key_file=None,
-      cert_file=None, master_ca_file=None, timeout=None):
+               cert_file=None, master_ca_file=None, timeout=None):
     self.connection_wrapper = connection_wrapper
     self.host = host
     self.path = path
@@ -599,8 +597,8 @@ class ConnectionHelper:
     self.timeout = timeout
 
   def getComputerInformation(self, computer_id):
-    self.GET('/getComputerInformation?computer_id=%s' % computer_id)
-    return xml_marshaller.loads(self.response.read())
+    xml = self.GET('/getComputerInformation?computer_id=%s' % computer_id)
+    return xml_marshaller.loads(xml)
 
   def getFullComputerInformation(self, computer_id):
     """
@@ -612,23 +610,23 @@ class ConnectionHelper:
       # XXX-Cedric: should raise something smarter than "NotFound".
       raise NotFoundError(method)
     try:
-      self.GET(method)
+      xml = self.GET(method)
     except NotFoundError:
       # XXX: This is a ugly way to keep backward compatibility,
       # We should stablise slap library soon.
-      self.GET('/getComputerInformation?computer_id=%s' % computer_id)
+      xml = self.GET('/getComputerInformation?computer_id=%s' % computer_id)
 
-    return xml_marshaller.loads(self.response.read())
+    return xml_marshaller.loads(xml)
 
   def connect(self):
-    connection_dict = dict(
-        host=self.host)
+    connection_dict = {
+            'host': self.host
+            }
     if self.key_file and self.cert_file:
-      connection_dict.update(
-        key_file=self.key_file,
-        cert_file=self.cert_file)
-    if self.master_ca_file is not None:
-      connection_dict.update(ca_file=self.master_ca_file)
+      connection_dict['key_file'] = self.key_file
+      connection_dict['cert_file'] = self.cert_file
+    if self.master_ca_file:
+      connection_dict['ca_file'] = self.master_ca_file
     self.connection = self.connection_wrapper(**connection_dict)
 
   def GET(self, path):
@@ -638,33 +636,37 @@ class ConnectionHelper:
       try:
         self.connect()
         self.connection.request('GET', self.path + path)
-        self.response = self.connection.getresponse()
+        response = self.connection.getresponse()
       # If ssl error : may come from bad configuration
-      except ssl.SSLError, e:
-        if e.message == "The read operation timed out":
-          raise socket.error(str(e) + self.error_message_timeout)
-        raise ssl.SSLError(str(e) + self.ssl_error_message_connect_fail)
-      except socket.error, e:
-        if e.message == "timed out":
-          raise socket.error(str(e) + self.error_message_timeout)
-        raise socket.error(self.error_message_connect_fail + str(e))
+      except ssl.SSLError as exc:
+        if exc.message == 'The read operation timed out':
+          raise socket.error(str(exc) + self.error_message_timeout)
+        raise ssl.SSLError(str(exc) + self.ssl_error_message_connect_fail)
+      except socket.error as exc:
+        if exc.message == 'timed out':
+          raise socket.error(str(exc) + self.error_message_timeout)
+        raise socket.error(self.error_message_connect_fail + str(exc))
+
       # check self.response.status and raise exception early
-      if self.response.status == httplib.REQUEST_TIMEOUT:
+      if response.status == httplib.REQUEST_TIMEOUT:
         # resource is not ready
         raise ResourceNotReady(path)
-      elif self.response.status == httplib.NOT_FOUND:
+      elif response.status == httplib.NOT_FOUND:
         raise NotFoundError(path)
-      elif self.response.status == httplib.FORBIDDEN:
+      elif response.status == httplib.FORBIDDEN:
         raise Unauthorized(path)
-      elif self.response.status != httplib.OK:
-        message = 'Server responded with wrong code %s with %s' % \
-                                           (self.response.status, path)
+      elif response.status != httplib.OK:
+        message = parsed_error_message(response.status,
+                                       response.read(),
+                                       path)
         raise ServerError(message)
     finally:
       socket.setdefaulttimeout(default_timeout)
 
+    return response.read()
+
   def POST(self, path, parameter_dict,
-      content_type="application/x-www-form-urlencoded"):
+           content_type='application/x-www-form-urlencoded'):
     try:
       default_timeout = socket.getdefaulttimeout()
       socket.setdefaulttimeout(self.timeout)
@@ -674,37 +676,39 @@ class ConnectionHelper:
         self.connection.request("POST", self.path + path,
             urllib.urlencode(parameter_dict), header_dict)
       # If ssl error : must come from bad configuration
-      except ssl.SSLError, e:
-        raise ssl.SSLError(self.ssl_error_message_connect_fail + str(e))
-      except socket.error, e:
-        raise socket.error(self.error_message_connect_fail + str(e))
-      self.response = self.connection.getresponse()
+      except ssl.SSLError as exc:
+        raise ssl.SSLError(self.ssl_error_message_connect_fail + str(exc))
+      except socket.error as exc:
+        raise socket.error(self.error_message_connect_fail + str(exc))
+
+      response = self.connection.getresponse()
       # check self.response.status and raise exception early
-      if self.response.status == httplib.REQUEST_TIMEOUT:
+      if response.status == httplib.REQUEST_TIMEOUT:
         # resource is not ready
         raise ResourceNotReady("%s - %s" % (path, parameter_dict))
-      elif self.response.status == httplib.NOT_FOUND:
+      elif response.status == httplib.NOT_FOUND:
         raise NotFoundError("%s - %s" % (path, parameter_dict))
-      elif self.response.status == httplib.FORBIDDEN:
+      elif response.status == httplib.FORBIDDEN:
         raise Unauthorized("%s - %s" % (path, parameter_dict))
-      elif self.response.status != httplib.OK:
-        message = 'Server responded with wrong code %s with %s' % \
-                                           (self.response.status, path)
+      elif response.status != httplib.OK:
+        message = parsed_error_message(response.status,
+                                       response.read(),
+                                       path)
         raise ServerError(message)
     finally:
       socket.setdefaulttimeout(default_timeout)
 
-class slap:
+    return response.read()
 
+
+class slap:
   zope.interface.implements(interface.slap)
 
   def initializeConnection(self, slapgrid_uri, key_file=None, cert_file=None,
-      master_ca_file=None, timeout=60):
-    scheme, netloc, path, query, fragment = urlparse.urlsplit(
-        slapgrid_uri)
-    if not(query == '' and fragment == ''):
-      raise AttributeError('Passed URL %r issue: not parseable'%
-          slapgrid_uri)
+                           master_ca_file=None, timeout=60):
+    scheme, netloc, path, query, fragment = urlparse.urlsplit(slapgrid_uri)
+    if not (query == '' and fragment == ''):
+      raise AttributeError('Passed URL %r issue: not parseable' % slapgrid_uri)
 
     if scheme == 'http':
       connection_wrapper = httplib.HTTPConnection
@@ -714,8 +718,8 @@ class slap:
       else:
         connection_wrapper = httplib.HTTPSConnection
     else:
-      raise AttributeError('Passed URL %r issue: there is no support for %r p'
-          'rotocol' % (slapgrid_uri, scheme))
+      raise AttributeError('Passed URL %r issue: there is no support '
+                           'for %r protocol' % (slapgrid_uri, scheme))
     self._connection_helper = ConnectionHelper(connection_wrapper,
           netloc, path, key_file, cert_file, master_ca_file, timeout)
 
@@ -726,7 +730,7 @@ class slap:
     returns SoftwareRelease class object
     """
     return SoftwareRelease(software_release=software_release,
-      connection_helper=self._connection_helper,
+      connection_helper=self._connection_helper
     )
 
   def registerComputer(self, computer_guid):
@@ -745,10 +749,10 @@ class slap:
       # XXX-Cedric: should raise something smarter than NotFound
       raise NotFoundError
 
-    self._connection_helper.GET('/registerComputerPartition?' \
+    xml = self._connection_helper.GET('/registerComputerPartition?' \
         'computer_reference=%s&computer_partition_reference=%s' % (
           computer_guid, partition_id))
-    result = xml_marshaller.loads(self._connection_helper.response.read())
+    result = xml_marshaller.loads(xml)
     # XXX: dirty hack to make computer partition usable. xml_marshaller is too
     # low-level for our needs here.
     result._connection_helper = self._connection_helper
@@ -759,3 +763,21 @@ class slap:
 
   def registerSupply(self):
     return Supply(connection_helper=self._connection_helper)
+
+  def getSoftwareReleaseListFromSoftwareProduct(self,
+      software_product_reference=None, software_release_url=None):
+    url = '/getSoftwareReleaseListFromSoftwareProduct?'
+    if software_product_reference:
+      if software_release_url is not None:
+        raise AttributeError('Both software_product_reference and '
+                             'software_release_url parameters are specified.')
+      url += 'software_product_reference=%s' % software_product_reference
+    else:
+      if software_release_url is None:
+        raise AttributeError('None of software_product_reference and '
+                             'software_release_url parameters are specified.')
+      url += 'software_release_url=%s' % software_release_url
+
+    result = xml_marshaller.loads(self._connection_helper.GET(url))
+    assert(type(result) == list)
+    return result
