@@ -45,9 +45,11 @@ USER_LIST = []
 GROUP_LIST = []
 INTERFACE_DICT = {}
 
+
 class FakeConfig:
   pass
 
+
 class TestLoggerHandler(logging.Handler):
   def __init__(self, *args, **kwargs):
     self.bucket = []
@@ -56,6 +58,7 @@ class TestLoggerHandler(logging.Handler):
   def emit(self, record):
     self.bucket.append(record.msg)
 
+
 class FakeCallAndRead:
   def __init__(self):
     self.external_command_list = []
@@ -83,11 +86,13 @@ class FakeCallAndRead:
         INTERFACE_DICT[interface][socket.AF_INET6].append({'addr': ip, 'netmask': netmask})
       # stabilise by mangling ip to just ip string
       argument_list[3] = 'ip/%s' % netmask
-    elif argument_list[:3] == ['ip', 'addr', 'list']:
+    elif argument_list[:3] == ['ip', 'addr', 'list'] or \
+         argument_list[:4] == ['ip', '-6', 'addr', 'list']:
       retval = 0, str(INTERFACE_DICT)
     self.external_command_list.append(' '.join(argument_list))
     return retval
 
+
 class LoggableWrapper:
   def __init__(self, logger, name):
     self.__logger = logger
@@ -98,11 +103,13 @@ class LoggableWrapper:
       '%s=%r' % (x, y) for x, y in kwargs.iteritems()]
     self.__logger.debug('%s(%s)' % (self.__name, ', '.join(arg_list)))
 
+
 class TimeMock:
   @classmethod
   def sleep(self, seconds):
     return
 
+
 class GrpMock:
   @classmethod
   def getgrnam(self, name):
@@ -111,6 +118,7 @@ class GrpMock:
       return True
     raise KeyError
 
+
 class PwdMock:
   @classmethod
   def getpwnam(self, name):
@@ -122,6 +130,7 @@ class PwdMock:
       return result
     raise KeyError
 
+
 class NetifacesMock:
   @classmethod
   def ifaddresses(self, name):
@@ -135,12 +144,14 @@ class NetifacesMock:
     global INTERFACE_DICT
     return INTERFACE_DICT.keys()
 
+
 class SlapformatMixin(unittest.TestCase):
   # keep big diffs
   maxDiff = None
+
   def patchNetifaces(self):
     self.netifaces = NetifacesMock()
-    self.saved_netifaces = dict()
+    self.saved_netifaces = {}
     for fake in vars(NetifacesMock):
       self.saved_netifaces[fake] = getattr(netifaces, fake, None)
       setattr(netifaces, fake, getattr(self.netifaces, fake))
@@ -151,7 +162,7 @@ class SlapformatMixin(unittest.TestCase):
     del self.saved_netifaces
 
   def patchPwd(self):
-    self.saved_pwd = dict()
+    self.saved_pwd = {}
     for fake in vars(PwdMock):
       self.saved_pwd[fake] = getattr(pwd, fake, None)
       setattr(pwd, fake, getattr(PwdMock, fake))
@@ -162,7 +173,7 @@ class SlapformatMixin(unittest.TestCase):
     del self.saved_pwd
 
   def patchTime(self):
-    self.saved_time = dict()
+    self.saved_time = {}
     for fake in vars(TimeMock):
       self.saved_time[fake] = getattr(time, fake, None)
       setattr(time, fake, getattr(TimeMock, fake))
@@ -173,7 +184,7 @@ class SlapformatMixin(unittest.TestCase):
     del self.saved_time
 
   def patchGrp(self):
-    self.saved_grp = dict()
+    self.saved_grp = {}
     for fake in vars(GrpMock):
       self.saved_grp[fake] = getattr(grp, fake, None)
       setattr(grp, fake, getattr(GrpMock, fake))
@@ -184,7 +195,7 @@ class SlapformatMixin(unittest.TestCase):
     del self.saved_grp
 
   def patchOs(self, logger):
-    self.saved_os = dict()
+    self.saved_os = {}
     for fake in ['mkdir', 'chown', 'chmod', 'makedirs']:
       self.saved_os[fake] = getattr(os, fake, None)
       f = LoggableWrapper(logger, fake)
@@ -230,6 +241,7 @@ class SlapformatMixin(unittest.TestCase):
     self.restoreNetifaces()
     slapos.format.callAndRead = self.real_callAndRead
 
+
 class TestComputer(SlapformatMixin):
   def test_getAddress_empty_computer(self):
     computer = slapos.format.Computer('computer')
@@ -243,7 +255,9 @@ class TestComputer(SlapformatMixin):
   @unittest.skip("Not implemented")
   def test_construct_empty_prepared(self):
     computer = slapos.format.Computer('computer',
-      interface=slapos.format.Interface('bridge', '127.0.0.1/16'))
+      interface=slapos.format.Interface(logger=self.test_result,
+                                        name='bridge',
+                                        ipv4_local_network='127.0.0.1/16'))
     computer.instance_root = '/instance_root'
     computer.software_root = '/software_root'
     computer.construct()
@@ -256,13 +270,15 @@ class TestComputer(SlapformatMixin):
     self.assertEqual([
       'ip addr list bridge',
       'groupadd slapsoft',
-      'useradd -d /software_root -g slapsoft -s /bin/false slapsoft -r'
+      'useradd -d /software_root -g slapsoft slapsoft -r'
       ],
       self.fakeCallAndRead.external_command_list)
 
   def test_construct_empty_prepared_no_alter_user(self):
     computer = slapos.format.Computer('computer',
-      interface=slapos.format.Interface('bridge', '127.0.0.1/16'))
+      interface=slapos.format.Interface(logger=self.test_result,
+                                        name='bridge',
+                                        ipv4_local_network='127.0.0.1/16'))
     computer.instance_root = '/instance_root'
     computer.software_root = '/software_root'
     computer.construct(alter_user=False)
@@ -271,14 +287,15 @@ class TestComputer(SlapformatMixin):
       "makedirs('/software_root', 493)",
       "chmod('/software_root', 493)"],
       self.test_result.bucket)
-    self.assertEqual([
-      'ip addr list bridge',],
-      self.fakeCallAndRead.external_command_list)
+    self.assertEqual(['ip addr list bridge'],
+                     self.fakeCallAndRead.external_command_list)
 
   @unittest.skip("Not implemented")
   def test_construct_empty_prepared_no_alter_network(self):
     computer = slapos.format.Computer('computer',
-      interface=slapos.format.Interface('bridge', '127.0.0.1/16'))
+      interface=slapos.format.Interface(logger=self.test_result,
+                                        name='bridge',
+                                        ipv4_local_network='127.0.0.1/16'))
     computer.instance_root = '/instance_root'
     computer.software_root = '/software_root'
     computer.construct(alter_network=False)
@@ -291,13 +308,15 @@ class TestComputer(SlapformatMixin):
     self.assertEqual([
       'ip addr list bridge',
       'groupadd slapsoft',
-      'useradd -d /software_root -g slapsoft -s /bin/false slapsoft -r'
+      'useradd -d /software_root -g slapsoft slapsoft -r'
       ],
       self.fakeCallAndRead.external_command_list)
 
   def test_construct_empty_prepared_no_alter_network_user(self):
     computer = slapos.format.Computer('computer',
-      interface=slapos.format.Interface('bridge', '127.0.0.1/16'))
+      interface=slapos.format.Interface(logger=self.test_result,
+                                        name='bridge',
+                                        ipv4_local_network='127.0.0.1/16'))
     computer.instance_root = '/instance_root'
     computer.software_root = '/software_root'
     computer.construct(alter_network=False, alter_user=False)
@@ -314,7 +333,9 @@ class TestComputer(SlapformatMixin):
   @unittest.skip("Not implemented")
   def test_construct_prepared(self):
     computer = slapos.format.Computer('computer',
-      interface=slapos.format.Interface('bridge', '127.0.0.1/16'))
+      interface=slapos.format.Interface(logger=self.test_result,
+                                        name='bridge',
+                                        ipv4_local_network='127.0.0.1/16'))
     computer.instance_root = '/instance_root'
     computer.software_root = '/software_root'
     partition = slapos.format.Partition('partition', '/part_path',
@@ -342,9 +363,9 @@ class TestComputer(SlapformatMixin):
     self.assertEqual([
       'ip addr list bridge',
       'groupadd slapsoft',
-      'useradd -d /software_root -g slapsoft -s /bin/false slapsoft -r',
+      'useradd -d /software_root -g slapsoft slapsoft -r',
       'groupadd testuser',
-      'useradd -d /instance_root/partition -g testuser -s /bin/false -G slapsoft testuser -r',
+      'useradd -d /instance_root/partition -g testuser -G slapsoft testuser -r',
       'tunctl -t tap -u testuser',
       'ip link set tap up',
       'brctl show',
@@ -358,13 +379,15 @@ class TestComputer(SlapformatMixin):
 
   def test_construct_prepared_no_alter_user(self):
     computer = slapos.format.Computer('computer',
-      interface=slapos.format.Interface('bridge', '127.0.0.1/16'))
+      interface=slapos.format.Interface(logger=self.test_result,
+                                        name='bridge',
+                                        ipv4_local_network='127.0.0.1/16'))
     computer.instance_root = '/instance_root'
     computer.software_root = '/software_root'
     partition = slapos.format.Partition('partition', '/part_path',
       slapos.format.User('testuser'), [], None)
     global USER_LIST
-    USER_LIST=['testuser']
+    USER_LIST = ['testuser']
     partition.tap = slapos.format.Tap('tap')
     computer.partition_list = [partition]
     global INTERFACE_DICT
@@ -392,14 +415,16 @@ class TestComputer(SlapformatMixin):
       'ip addr add ip/255.255.255.255 dev bridge',
       # 'ip addr list bridge',
       'ip addr add ip/ffff:ffff:ffff:ffff:: dev bridge',
-      'ip addr list bridge',
+      'ip -6 addr list bridge',
     ],
       self.fakeCallAndRead.external_command_list)
 
   @unittest.skip("Not implemented")
   def test_construct_prepared_no_alter_network(self):
     computer = slapos.format.Computer('computer',
-      interface=slapos.format.Interface('bridge', '127.0.0.1/16'))
+      interface=slapos.format.Interface(logger=self.test_result,
+                                        name='bridge',
+                                        ipv4_local_network='127.0.0.1/16'))
     computer.instance_root = '/instance_root'
     computer.software_root = '/software_root'
     partition = slapos.format.Partition('partition', '/part_path',
@@ -427,9 +452,9 @@ class TestComputer(SlapformatMixin):
     self.assertEqual([
       # 'ip addr list bridge',
       'groupadd slapsoft',
-      'useradd -d /software_root -g slapsoft -s /bin/false slapsoft -r',
+      'useradd -d /software_root -g slapsoft slapsoft -r',
       'groupadd testuser',
-      'useradd -d /instance_root/partition -g testuser -s /bin/false -G slapsoft testuser -r',
+      'useradd -d /instance_root/partition -g testuser -G slapsoft testuser -r',
       # 'ip addr add ip/255.255.255.255 dev bridge',
       # 'ip addr list bridge',
       # 'ip addr add ip/ffff:ffff:ffff:ffff:: dev bridge',
@@ -439,7 +464,9 @@ class TestComputer(SlapformatMixin):
 
   def test_construct_prepared_no_alter_network_user(self):
     computer = slapos.format.Computer('computer',
-      interface=slapos.format.Interface('bridge', '127.0.0.1/16'))
+      interface=slapos.format.Interface(logger=self.test_result,
+                                        name='bridge',
+                                        ipv4_local_network='127.0.0.1/16'))
     computer.instance_root = '/instance_root'
     computer.software_root = '/software_root'
     partition = slapos.format.Partition('partition', '/part_path',
@@ -467,24 +494,12 @@ class TestComputer(SlapformatMixin):
       'ip addr add ip/255.255.255.255 dev bridge',
       # 'ip addr list bridge',
       'ip addr add ip/ffff:ffff:ffff:ffff:: dev bridge',
-      'ip addr list bridge',
+      'ip -6 addr list bridge',
     ],
       self.fakeCallAndRead.external_command_list)
 
-class TestPartition(SlapformatMixin):
 
-  def test_createPath(self):
-    global USER_LIST
-    USER_LIST = ['testuser']
-    self.partition.createPath()
-    self.assertEqual(
-      [
-        "mkdir('/part_path', 488)",
-        "chown('/part_path', 0, 0)",
-        "chmod('/part_path', 488)"
-      ],
-      self.test_result.bucket
-    )
+class TestPartition(SlapformatMixin):
 
   def test_createPath_no_alter_user(self):
     self.partition.createPath(False)
@@ -496,6 +511,7 @@ class TestPartition(SlapformatMixin):
       self.test_result.bucket
     )
 
+
 class TestUser(SlapformatMixin):
   def test_create(self):
     user = slapos.format.User('doesnotexistsyet')
@@ -504,7 +520,7 @@ class TestUser(SlapformatMixin):
 
     self.assertEqual([
       'groupadd doesnotexistsyet',
-      'useradd -d /doesnotexistsyet -g doesnotexistsyet -s /bin/false '\
+      'useradd -d /doesnotexistsyet -g doesnotexistsyet '\
         'doesnotexistsyet -r'
     ],
       self.fakeCallAndRead.external_command_list)
@@ -517,7 +533,7 @@ class TestUser(SlapformatMixin):
 
     self.assertEqual([
       'groupadd doesnotexistsyet',
-      'useradd -d /doesnotexistsyet -g doesnotexistsyet -s /bin/false -G '\
+      'useradd -d /doesnotexistsyet -g doesnotexistsyet -G '\
         'additionalgroup1,additionalgroup2 doesnotexistsyet -r'
       ],
       self.fakeCallAndRead.external_command_list)
@@ -530,7 +546,7 @@ class TestUser(SlapformatMixin):
     user.create()
 
     self.assertEqual([
-      'useradd -d /testuser -g testuser -s /bin/false testuser -r'
+      'useradd -d /testuser -g testuser testuser -r'
     ],
       self.fakeCallAndRead.external_command_list)
 
@@ -544,7 +560,7 @@ class TestUser(SlapformatMixin):
 
     self.assertEqual([
       'groupadd testuser',
-      'usermod -d /testuser -g testuser -s /bin/false -G '\
+      'usermod -d /testuser -g testuser -G '\
         'additionalgroup1,additionalgroup2 testuser'
     ],
       self.fakeCallAndRead.external_command_list)
@@ -558,7 +574,7 @@ class TestUser(SlapformatMixin):
 
     self.assertEqual([
       'groupadd testuser',
-      'usermod -d /testuser -g testuser -s /bin/false testuser'
+      'usermod -d /testuser -g testuser testuser'
     ],
       self.fakeCallAndRead.external_command_list)
 
@@ -572,7 +588,7 @@ class TestUser(SlapformatMixin):
     user.create()
 
     self.assertEqual([
-      'usermod -d /testuser -g testuser -s /bin/false testuser'
+      'usermod -d /testuser -g testuser testuser'
     ],
       self.fakeCallAndRead.external_command_list)
 
